@@ -13,6 +13,18 @@
 #include "proctable.h"
 
 
+#define APP_ICON_SIZE 16
+
+
+struct _PrettyTable {
+	GHashTable	*app_hash;	/* WNCK */
+	GHashTable	*default_hash;	/* defined in defaulttable.h */
+	gchar		*datadir;
+	GtkIconTheme	*theme;
+};
+
+
+
 static void
 load_default_table(PrettyTable *pretty_table,
 		   const PrettyTableItem table[], size_t n);
@@ -51,7 +63,7 @@ new_application (WnckScreen *screen, WnckApplication *app, gpointer data)
 	if (!tmp)
 		return;
 
-	icon =  gdk_pixbuf_scale_simple (tmp, 16, 16, GDK_INTERP_HYPER);
+	icon =  gdk_pixbuf_scale_simple (tmp, APP_ICON_SIZE, APP_ICON_SIZE, GDK_INTERP_HYPER);
 	if (!icon)
 		return;
 
@@ -64,6 +76,7 @@ new_application (WnckScreen *screen, WnckApplication *app, gpointer data)
 		if (info->pixbuf)
 			return;
 		info->pixbuf = icon;
+		g_object_ref(info->pixbuf);
 		if (info->is_visible) {
 			GtkTreeModel *model;
 			model = gtk_tree_view_get_model (GTK_TREE_VIEW (procdata->tree));
@@ -72,6 +85,7 @@ new_application (WnckScreen *screen, WnckApplication *app, gpointer data)
 		}
 	}
 
+	g_assert(g_hash_table_lookup(hash, GUINT_TO_POINTER(pid)) == NULL);
 	g_hash_table_insert (hash, GUINT_TO_POINTER(pid), icon);
 }
 
@@ -92,7 +106,7 @@ application_finished (WnckScreen *screen, WnckApplication *app, gpointer data)
 					  &orig_pid, &icon)) {
 		g_hash_table_remove (hash, orig_pid);
 		if (icon)
-			g_object_unref (G_OBJECT (icon));
+			g_object_unref (icon);
 	}
 }
 
@@ -102,7 +116,7 @@ void pretty_table_new (ProcData *procdata)
 	WnckScreen *screen;
 	PrettyTable *pretty_table = g_new(PrettyTable, 1);
 
-	pretty_table->app_hash = g_hash_table_new (g_direct_hash, g_direct_equal);
+	pretty_table->app_hash = g_hash_table_new (NULL, NULL);
 	pretty_table->default_hash = g_hash_table_new (g_str_hash, g_str_equal);
 
 	screen = wnck_screen_get_default ();
@@ -120,7 +134,63 @@ void pretty_table_new (ProcData *procdata)
 
 	procdata->pretty_table = pretty_table;
 
+	pretty_table->theme = gtk_icon_theme_get_default ();
+}
 
+
+
+
+
+static GdkPixbuf *
+get_icon_from_theme (PrettyTable *pretty_table,
+		     guint pid,
+		     const gchar *command)
+{
+	return gtk_icon_theme_load_icon (pretty_table->theme,
+					 command,
+					 APP_ICON_SIZE,
+					 0,
+					 NULL);
+}
+
+
+
+static GdkPixbuf *
+get_icon_from_default (PrettyTable *pretty_table,
+		       guint pid,
+		       const gchar *command)
+{
+	GdkPixbuf *icon;
+
+	icon = g_hash_table_lookup (pretty_table->default_hash,
+				    command);
+
+	if (icon) {
+		g_object_ref (icon);
+		return icon;
+	}
+
+	return NULL;
+}
+
+
+
+static GdkPixbuf *
+get_icon_from_wnck (PrettyTable *pretty_table,
+		    guint pid,
+		    const gchar *command)
+{
+	GdkPixbuf *icon;
+
+	icon = g_hash_table_lookup (pretty_table->app_hash,
+				    GUINT_TO_POINTER(pid));
+
+	if (icon) {
+		g_object_ref (icon);
+		return icon;
+	}
+
+	return NULL;
 }
 
 
@@ -128,16 +198,34 @@ void pretty_table_new (ProcData *procdata)
 GdkPixbuf *pretty_table_get_icon (PrettyTable *pretty_table,
 				  const gchar *command, guint pid)
 {
-	GdkPixbuf *icon;
+	typedef GdkPixbuf * (*IconGetter) (PrettyTable *pretty_table,
+					   guint pid,
+					   const gchar *command);
 
-	icon = g_hash_table_lookup (pretty_table->default_hash, command);
 
-	if (icon) {
-		return icon;
+	const IconGetter getters[] = {
+		get_icon_from_theme,
+		get_icon_from_wnck,
+		get_icon_from_default
+	};
+
+
+	size_t i;
+
+	for (i = 0; i < G_N_ELEMENTS (getters); ++i) {
+		GdkPixbuf *icon;
+
+		icon = (getters[i]) (pretty_table, pid, command);
+
+		if (icon) {
+			g_print("Get icon method %lu for '%s'\n",
+				(gulong)i,
+				command);
+			return icon;
+		}
 	}
 
-	icon = g_hash_table_lookup (pretty_table->app_hash, GUINT_TO_POINTER(pid));
-	return icon;
+	return NULL;
 }
 
 
@@ -162,7 +250,7 @@ create_scaled_icon(const char *iconpath)
 	GdkPixbuf *scaled;
 
 	scaled = gdk_pixbuf_new_from_file_at_scale(iconpath,
-						   16, 16,
+						   APP_ICON_SIZE, APP_ICON_SIZE,
 						   TRUE,
 						   &error);
 
@@ -199,13 +287,13 @@ load_icon_for_commands(gpointer key, gpointer value, gpointer userdata)
 
 		for(i = 0; i < commands->len; ++i)
 		{
-			gdk_pixbuf_ref(scaled);
+			g_object_ref(scaled);
 			g_hash_table_insert(pretty_table->default_hash,
 					    g_strdup(g_ptr_array_index(commands, i)),
 					    scaled);
 		}
 
-		gdk_pixbuf_unref(scaled);
+		g_object_unref(scaled);
 	}
 }
 
